@@ -1,6 +1,7 @@
 /*
  * osmo-pcap-server code
  *
+ * (C) 2024 by sysmocom - s.f.m.c. GmbH <info@sysmocom.de>
  * (C) 2011-2016 by Holger Hans Peter Freyther <holger@moiji-mobile.com>
  * (C) 2011 by On-Waves
  * All Rights Reserved
@@ -52,46 +53,11 @@ static int daemonize = 0;
 
 void *tall_srv_ctx;
 struct osmo_pcap_server *pcap_server;
-extern void *tall_msgb_ctx;
-extern void *tall_ctr_ctx;
-
-
-static const struct rate_ctr_desc pcap_peer_ctr_desc[] = {
-	[PEER_CTR_CONNECT]		= { "peer:connect",	"Connect of a peer   " },
-	[PEER_CTR_BYTES]		= { "peer:bytes",	"Received bytes      " },
-	[PEER_CTR_PKTS]			= { "peer:pkts",	"Received packets    " },
-	[PEER_CTR_PROTATE]		= { "peer:file_rotated","Capture file rotated" },
-};
-
-static const struct rate_ctr_desc pcap_server_ctr_desc[] = {
-	[SERVER_CTR_CONNECT]		= { "server:connect",	"Connect of a peer   " },
-	[SERVER_CTR_BYTES]		= { "server:bytes",	"Received bytes      " },
-	[SERVER_CTR_PKTS]		= { "server:pkts",	"Received packets    " },
-	[SERVER_CTR_PROTATE]		= { "server:file_rotated", "Capture file rotated" },
-	[SERVER_CTR_NOCLIENT]		= { "server:no_client", "Unknown connected   " },
-};
-
-const struct rate_ctr_group_desc pcap_peer_group_desc = {
-	.group_name_prefix		= NULL,	/* will be dynamically patched */
-	.group_description		= "PCAP peer statistics",
-	.num_ctr			= ARRAY_SIZE(pcap_peer_ctr_desc),
-	.ctr_desc			= pcap_peer_ctr_desc,
-	.class_id			= OSMO_STATS_CLASS_PEER,
-};
-
-static const struct rate_ctr_group_desc pcap_server_group_desc = {
-	.group_name_prefix		= "pcap:server",
-	.group_description		= "PCAP Server global statistics",
-	.num_ctr			= ARRAY_SIZE(pcap_server_ctr_desc),
-	.ctr_desc			= pcap_server_ctr_desc,
-	.class_id			= OSMO_STATS_CLASS_GLOBAL,
-};
 
 static struct vty_app_info vty_info = {
 	.name		= "OsmoPCAPServer",
 	.version	= PACKAGE_VERSION,
 	.go_parent_cb	= osmopcap_go_parent,
-	.is_config_node	= osmopcap_is_config_node,
 };
 
 static void print_usage()
@@ -235,8 +201,7 @@ static void signal_handler(int signum)
 static void talloc_init_ctx()
 {
 	tall_srv_ctx = talloc_named_const(NULL, 0, "server");
-	tall_msgb_ctx = talloc_named_const(tall_srv_ctx, 0, "msgb");
-	tall_ctr_ctx = talloc_named_const(tall_srv_ctx, 0, "counter");
+	msgb_talloc_ctx_init(tall_srv_ctx, 0);
 }
 
 int main(int argc, char **argv)
@@ -285,8 +250,13 @@ int main(int argc, char **argv)
 	INIT_LLIST_HEAD(&pcap_server->conn);
 	pcap_server->base_path = talloc_strdup(pcap_server, "./");
 	pcap_server->permission_mask = 0440;
-	pcap_server->max_size = 1073741824;
+	pcap_server->max_size = 1073741824; /* 1024^3, 1GB **/
+	pcap_server->max_size_enabled = true;
 	pcap_server->max_snaplen = DEFAULT_SNAPLEN;
+	/* By default rotate daily: */
+	pcap_server->rotate_localtime.enabled = true;
+	pcap_server->rotate_localtime.intv = TIME_INTERVAL_DAY;
+	pcap_server->rotate_localtime.modulus = 1;
 
 	if (vty_read_config_file(config_file, NULL) < 0) {
 		LOGP(DSERVER, LOGL_ERROR,
@@ -294,7 +264,7 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
-	rc = telnet_init_dynif(tall_srv_ctx, NULL, vty_get_bind_addr(), OSMO_VTY_PORT_PCAP_SERVER);
+	rc = telnet_init_default(tall_srv_ctx, NULL, OSMO_VTY_PORT_PCAP_SERVER);
 	if (rc < 0) {
 		LOGP(DCLIENT, LOGL_ERROR, "Failed to bind telnet interface\n");
 		exit(1);
