@@ -1,6 +1,7 @@
 /*
  * osmo-pcap-server code
  *
+ * (C) 2024 by sysmocom - s.f.m.c. GmbH <info@sysmocom.de>
  * (C) 2011-2016 by Holger Hans Peter Freyther <holger@moiji-mobile.com>
  * (C) 2011 by On-Waves
  * All Rights Reserved
@@ -130,10 +131,10 @@ void osmo_pcap_server_close_trace(struct osmo_pcap_conn *conn)
 static void close_connection(struct osmo_pcap_conn *conn)
 {
 	if (conn->rem_wq.bfd.fd >= 0) {
+		osmo_fd_unregister(&conn->rem_wq.bfd);
 		close(conn->rem_wq.bfd.fd);
 		conn->rem_wq.bfd.fd = -1;
 		osmo_tls_release(&conn->tls_session);
-		osmo_fd_unregister(&conn->rem_wq.bfd);
 	}
 
 	osmo_pcap_server_close_trace(conn);
@@ -173,7 +174,8 @@ static void restart_pcap(struct osmo_pcap_conn *conn)
 
 	conn->local_fd = creat(conn->curr_filename, conn->server->permission_mask);
 	if (conn->local_fd < 0) {
-		LOGP(DSERVER, LOGL_ERROR, "Failed to file: '%s'\n", conn->curr_filename);
+		LOGP(DSERVER, LOGL_ERROR, "Failed to create file '%s': %s\n",
+		     conn->curr_filename, strerror(errno));
 		return;
 	}
 
@@ -212,6 +214,141 @@ static int link_data(struct osmo_pcap_conn *conn, struct osmo_pcap_data *data)
 	return 1;
 }
 
+/* Returns true if pcap was re-opened */
+static bool check_restart_pcap_max_size(struct osmo_pcap_conn *conn, const struct osmo_pcap_data *data)
+{
+	off_t cur;
+
+	if (!pcap_server->max_size_enabled)
+		return false;
+	cur = lseek(conn->local_fd, 0, SEEK_CUR);
+	if (cur + data->len <= conn->server->max_size)
+		return false;
+	LOGP(DSERVER, LOGL_NOTICE, "Rolling over file for %s (max-size)\n", conn->name);
+	restart_pcap(conn);
+	return true;
+}
+
+/* Checks if we are in a new record period since last time we wrote to a pcap, to know (return true)
+ * whether a new pcap file needs to be opened. It calculates period based on intv and mod.
+ * NOTE: Kept non-static to be able to validate it with unit tests. */
+bool check_localtime(const struct tm *last_write, const struct tm *tm, enum time_interval intv, unsigned int mod)
+{
+	unsigned long long last_start, current_start;
+
+	switch (intv) {
+	case TIME_INTERVAL_SEC:
+		if (last_write->tm_sec == tm->tm_sec &&
+		    last_write->tm_min == tm->tm_min &&
+		    last_write->tm_hour == tm->tm_hour &&
+		    last_write->tm_mday == tm->tm_mday &&
+		    last_write->tm_mon == tm->tm_mon &&
+		    last_write->tm_year == tm->tm_year)
+			return false;
+		/* If minute/hour/day/month/year changed (passed through second 0), always rotate: */
+		if (last_write->tm_min < tm->tm_min ||
+		    last_write->tm_hour < tm->tm_hour ||
+		    last_write->tm_mday < tm->tm_mday ||
+		    last_write->tm_mon < tm->tm_mon ||
+		    last_write->tm_year < tm->tm_year)
+			return true;
+		/* Same minute/hour/day/month/year, second changed. Check if we are still in same period: */
+		last_start = last_write->tm_sec - (last_write->tm_sec % mod);
+		current_start = tm->tm_sec - (tm->tm_sec % mod);
+		if (current_start <= last_start)
+			return false;
+		return true;
+	case TIME_INTERVAL_MIN:
+		if (last_write->tm_min == tm->tm_min &&
+		    last_write->tm_hour == tm->tm_hour &&
+		    last_write->tm_mday == tm->tm_mday &&
+		    last_write->tm_mon == tm->tm_mon &&
+		    last_write->tm_year == tm->tm_year)
+			return false;
+		/* If hour/day/month/year changed (passed through minute 0), always rotate: */
+		if (last_write->tm_hour < tm->tm_hour ||
+		    last_write->tm_mday < tm->tm_mday ||
+		    last_write->tm_mon < tm->tm_mon ||
+		    last_write->tm_year < tm->tm_year)
+			return true;
+		/* Same hour/day/month/year, minute changed. Check if we are still in same period: */
+		last_start = last_write->tm_min - (last_write->tm_min % mod);
+		current_start = tm->tm_min - (tm->tm_min % mod);
+		if (current_start <= last_start)
+			return false;
+		return true;
+	case TIME_INTERVAL_HOUR:
+		if (last_write->tm_hour == tm->tm_hour &&
+		    last_write->tm_mday == tm->tm_mday &&
+		    last_write->tm_mon == tm->tm_mon &&
+		    last_write->tm_year == tm->tm_year)
+			return false;
+		/* If day/month/year changed (passed through hour 0), always rotate: */
+		if (last_write->tm_mday < tm->tm_mday ||
+		    last_write->tm_mon < tm->tm_mon ||
+		    last_write->tm_year < tm->tm_year)
+			return true;
+		/* Same day/month/year, hour changed. Check if we are still in same period: */
+		last_start = last_write->tm_hour - (last_write->tm_hour % mod);
+		current_start = tm->tm_hour - (tm->tm_hour % mod);
+		if (current_start <= last_start)
+			return false;
+		return true;
+	case TIME_INTERVAL_DAY:
+		if (last_write->tm_mday == tm->tm_mday &&
+		    last_write->tm_mon == tm->tm_mon &&
+		    last_write->tm_year == tm->tm_year)
+			return false;
+		/* If month/year changed (passed through day 1), always rotate: */
+		if (last_write->tm_mon < tm->tm_mon ||
+		    last_write->tm_year < tm->tm_year)
+			return true;
+		/* Same month/year, day changed. Check if we are still in same period: */
+		/* Note: tm_mday is [1, 31], hence the -1 below: */
+		last_start = (last_write->tm_mday - 1) - ((last_write->tm_mday - 1) % mod);
+		current_start = (tm->tm_mday - 1) - ((tm->tm_mday - 1) % mod);
+		if (current_start <= last_start)
+			return false;
+		return true;
+	case TIME_INTERVAL_MONTH:
+		if (last_write->tm_mon == tm->tm_mon &&
+		    last_write->tm_year == tm->tm_year)
+			return false;
+		/* If year changed (passed through month 1), always rotate: */
+		if (last_write->tm_year < tm->tm_year)
+			return true;
+		/* Same year, month changed. Check if we are still in same period: */
+		last_start = last_write->tm_mon - (last_write->tm_mon % mod);
+		current_start = tm->tm_mon - (tm->tm_mon % mod);
+		if (current_start <= last_start)
+			return false;
+		return true;
+	case TIME_INTERVAL_YEAR:
+		/* Year changed. Check if we are still in same period: */
+		last_start = last_write->tm_year - (last_write->tm_year % mod);
+		current_start = tm->tm_year - (tm->tm_year % mod);
+		if (current_start <= last_start)
+			return false;
+		return true;
+	default:
+		OSMO_ASSERT(false);
+	}
+}
+
+static bool check_restart_pcap_localtime(struct osmo_pcap_conn *conn, const struct tm *tm)
+{
+	if (!pcap_server->rotate_localtime.enabled)
+		return false;
+
+	if (!check_localtime(&conn->last_write, tm,
+			     pcap_server->rotate_localtime.intv,
+			     pcap_server->rotate_localtime.modulus))
+		return false;
+	LOGP(DSERVER, LOGL_NOTICE, "Rolling over file for %s (localtime)\n", conn->name);
+	restart_pcap(conn);
+	return true;
+}
+
 /*
  * Check if we are past the limit or on a day change
  */
@@ -233,16 +370,8 @@ static int write_data(struct osmo_pcap_conn *conn, struct osmo_pcap_data *data)
 		return -1;
 	}
 
-	off_t cur = lseek(conn->local_fd, 0, SEEK_CUR);
-	if (cur > conn->server->max_size) {
-		LOGP(DSERVER, LOGL_NOTICE, "Rolling over file for %s\n", conn->name);
-		restart_pcap(conn);
-	} else if (conn->last_write.tm_mday != tm->tm_mday ||
-		   conn->last_write.tm_mon != tm->tm_mon ||
-		   conn->last_write.tm_year != tm->tm_year) {
-		LOGP(DSERVER, LOGL_NOTICE, "Rolling over file for %s\n", conn->name);
-		restart_pcap(conn);
-	}
+	if (!check_restart_pcap_max_size(conn, data))
+		check_restart_pcap_localtime(conn, tm);
 
 	conn->last_write = *tm;
 	rc = write(conn->local_fd, &data->data[0], data->len);
